; EXPLOSIONS.ASM
; Drawing, updating of explosions.
; Most functions have a _sprites and a _terrain version.
; Make sure you are calling the right one!

;==============================================================
; DRAW EXPLOSIONS
;==============================================================

draw_explosions_sprites:
	ld hl,explosions_sprites
	ld b,max_explosions_sprites
	jr draw_explosions
draw_explosions_terrain:
	ld hl,explosions_terrain
	ld b,max_explosions_terrain
draw_explosions:
-:	push bc
	ld a,(hl)
	ld b,a
	inc hl
	ld a,(hl)
	ld c,a
	inc hl
	or b
	jr z,+
	ld a,(hl)
	ld d,a
	inc hl
	inc hl
	push hl
	call draw_large_sprite
	pop hl
	pop bc
	djnz -
	ret

+:	inc hl
	inc hl
	pop bc
	djnz -
	ret

;==============================================================
; ADD AN EXPLOSION AT LOCATION (B,C)
;==============================================================

add_explosion_sprites:
	push bc
	pop de
	ld hl,explosions_sprites
	ld b,max_explosions_sprites
	jr add_explosion

add_explosion_terrain:
	push bc
	pop de
	ld hl,explosions_terrain
	ld b,max_explosions_terrain
add_explosion:

-:	push bc
	ld a,(hl)
	ld b,a
	inc hl
	ld a,(hl)
	or b
	jr z,+
	inc hl
	inc hl
	inc hl
	pop bc
	djnz -

	ret

+:	pop bc
	inc hl
	ld a,56
	ld (hl),a
	inc hl
	xor a
	ld (hl),a
	dec hl
	dec hl
	ld a,e
	ld (hl),a
	dec hl
	ld a,d
	ld (hl),a
	ret

;==============================================================
; RUN THE EXPLOSION ANIMATION LOOPS
;==============================================================

animate_explosions_sprites:
	ld hl,explosions_sprites
	ld b,max_explosions_sprites
	ld c,0
	jr animate_explosions
animate_explosions_terrain:
	ld hl,explosions_terrain
	ld b,max_explosions_terrain
	ld c,1
animate_explosions:
-:	ld a,(hl)
	inc hl	; y
	or (hl)
	jr z,+

	ld a,(game_clock)
	ld d,a
	ld a,(land_speed)
	and d
	jr nz,++
	ld a,(hl)
	inc a
	ld (hl),a
++:

	inc hl	; s
	inc hl	; d

	ld a,c
	or a
	jr z,animate_sprite_explosions

	ld a,(hl)
	inc a
	ld (hl),a

	cp 10
	jr z,exp_set_medium
	cp 20
	jr z,exp_set_large
	cp 30
	jr z,exp_set_medium
	cp 40
	jr z,exp_set_small
	cp 50
	jr z,exp_clear

	jr done_animation


animate_sprite_explosions:
	ld a,(hl)
	inc a
	ld (hl),a

	cp 15
	jr z,exp_set_small
	cp 30
	jr z,exp_set_large
	cp 40
	jr z,exp_set_small
	cp 50
	jr z,exp_clear


done_animation:

	inc hl	; x
	djnz -
	ret

exp_set_large:
	dec hl	; s
	ld a,56
	ld (hl),a
	dec hl	; y
	jr +	
exp_set_small:
	dec hl	; s
	ld a,60
	ld (hl),a
	dec hl	; y
	jr +
exp_set_medium:
	dec hl	; s
	ld a,64
	ld (hl),a
	dec hl	; y
	jr +


exp_clear:
	xor a
	dec hl	; s
	dec hl	; y
	ld (hl),a
	dec hl	; x
	ld (hl),a
	inc hl	; y
	jr +

+:
	inc hl ; s
	inc hl ; d
	inc hl ; NEXT x
	djnz -
	ret